﻿using System;
using Microsoft.Extensions.Caching.Distributed;
using Falcon.Extend;

namespace Falcon.Extend
{
    /// <summary>
    /// 使用微软Redis扩展方法组件实现Redis缓冲
    /// </summary>
    public class RedisCacheProvider:ICacheProvider
    {
        /// <summary>
        /// 基础缓冲组件
        /// </summary>
        public IDistributedCache Cache { get; set; }
        /// <summary>
        /// 基础序列化组件
        /// </summary>
        public IJsonProvider Json { get; set; }
        /// <summary>
        /// 通过提供json序列化组件和基础缓冲组件实现Redis提供器
        /// </summary>
        /// <param name="jsonProvider">基础json序列化器</param>
        /// <param name="distributedCache">基础缓冲器</param>
        public RedisCacheProvider(IJsonProvider jsonProvider,IDistributedCache distributedCache = null) {
            this.Cache = distributedCache;
            this.Json = jsonProvider;
        }
        /// <summary>
        /// 通过key获取缓存对象
        /// </summary>
        /// <typeparam name="T">对象类型</typeparam>
        /// <param name="key">存储对象的key</param>
        /// <returns>缓存的对象</returns>
        public T GetObj<T>(string key) where T : class, new() {
            if(this.Cache == null) {
                return null;
            }
            var str = this.Cache.GetString(key);
            if(str.IsNullOrEmpty()) {
                return null;
            }
            return this.Json.GetObj<T>(str);
        }
        /// <summary>
        /// 存储对象
        /// </summary>
        /// <typeparam name="T">对象的类型</typeparam>
        /// <param name="key">缓存的key</param>
        /// <param name="obj">要缓存的对象</param>
        /// <param name="span">缓存时长</param>
        public void SetCache<T>(string key,T obj,TimeSpan span) where T : class {
            if(this.Cache == null || obj == null || span == null || span == TimeSpan.Zero) {
                return;
            }
            this.Cache.SetString(key,this.Json.GetJson(obj),new DistributedCacheEntryOptions {
                AbsoluteExpirationRelativeToNow = span,
            });
        }
    }
}
